# -*- coding: utf-8 -*-

from .service_config import ServiceConfig


class DbConfig(ServiceConfig):
    def __init__(self, parsed_yaml: {}, key: str):
        super().__init__(parsed_yaml, key)

        configs = parsed_yaml.get(key, {})
        self.host = configs.get('host', '')
        self.port = configs.get('port', '')
        self.user = configs.get('user', '')
        self.database = configs.get('database', '')



# -*- coding: utf-8 -*-

from .config import ConfigParser, ServiceConfig
from .exec_env import DisableEnv, Shell, Docker, StringBuilder
from .service import Git, DB, PackageManager
from .runner import InvokeRunner


class Container(object):
    def __init__(self, role: str):
        config = ConfigParser.find(role)

        self.string_builder = StringBuilder(config.string_builder, config.variable, InvokeRunner)
        self.shell = Shell(config.shell, config.variable, InvokeRunner)
        self.docker = Docker(config.docker, config.variable, InvokeRunner)

        self.git = Git(self.env(config.git, config.variable), config.git)
        self.db = DB(self.env(config.db, config.variable), config.db)
        self.php = PackageManager(self.env(config.php, config.variable), config.php)
        self.ruby = PackageManager(self.env(config.ruby, config.variable), config.ruby)
        self.node = PackageManager(self.env(config.node, config.variable), config.node)

        self.package_managers = [
            self.php,
            self.ruby,
            self.node
        ]

    @staticmethod
    def env(config: ServiceConfig, variable: {}):
        if config.env.name == 'string_builder' and config.env.work_dir != '':
            return StringBuilder(config.env, variable, InvokeRunner)
        elif config.env.name == 'shell' and config.env.work_dir != '':
            return Shell(config.env, variable, InvokeRunner)
        elif config.env.name == 'docker' and config.env.work_dir != '':
            return Docker(config.env, variable, InvokeRunner)
        else:
            return DisableEnv()

    @classmethod
    def all_role_names(cls):
        return ConfigParser.all_roles()



# -*- coding: utf-8 -*-


class ExecEnvConfig(object):
    def __init__(self, parsed_yaml: {}, key: str):
        configs = parsed_yaml.get(key, {})

        self.name = configs.get('name', '')
        self.work_dir = configs.get('work_dir', '')
        self.run = configs.get('run', '')



# -*- coding: utf-8 -*-

from .exec_env_config import ExecEnvConfig


class DockerConfig(ExecEnvConfig):
    def __init__(self, parsed_yaml: {}, key: str):
        super().__init__(parsed_yaml, key)

        configs = parsed_yaml.get(key, {})

        self.container = configs.get('container', '')
        self.ps = configs.get('ps', '')
        self.build = configs.get('build', '')
        self.up = configs.get('up', '')
        self.down = configs.get('down', '')



import unittest
from kumaoche.config import DbConfig


class TestDbConfig(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'db': {
                'run': '0',
                'setup': '1',
                'update': '2',
                'env': {
                    'name': '2.0',
                    'work_dir': '2.1',
                    'run': '2.2',
                },
                'host': '3',
                'port': '4',
                'user': '5',
                'database': '6',
            }
        }
        config = DbConfig(parsed_yaml, 'db')
        self.assertEqual('0', config.run)
        self.assertEqual('1', config.setup)
        self.assertEqual('2', config.update)
        self.assertEqual('2.0', config.env.name)
        self.assertEqual('2.1', config.env.work_dir)
        self.assertEqual('2.2', config.env.run)
        self.assertEqual('3', config.host)
        self.assertEqual('4', config.port)
        self.assertEqual('5', config.user)
        self.assertEqual('6', config.database)

        config = DbConfig({}, 'db')
        self.assertEqual('', config.run)
        self.assertEqual('', config.setup)
        self.assertEqual('', config.update)
        self.assertEqual('', config.env.name)
        self.assertEqual('', config.env.work_dir)
        self.assertEqual('', config.env.run)
        self.assertEqual('', config.host)
        self.assertEqual('', config.port)
        self.assertEqual('', config.user)
        self.assertEqual('', config.database)



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from ..config import GitConfig
from ..exec_env import ExecEnv
from .service import Service


class Git(Service):
    def __init__(self, env: ExecEnv, config: GitConfig):
        self.__env = env
        self.__config = config

    def run(self, command: str):
        repo_dir = self.__env.var_assign(self.__config.repo_dir)
        var_command = {'command': self.__env.var_assign(command)}

        return self.__env.run(self.__env.var_assign(self.__config.run, var_command), work_dir=repo_dir)

    def setup(self):
        return self.__env.run(self.__env.var_assign(self.__config.setup))

    def update(self):
        return self.__env.run(self.__env.var_assign(self.__config.update))



# -*- coding: utf-8 -*-

import yaml
import os
import sys

from .config_pack import ConfigPack


class ConfigParser(object):

    @classmethod
    def file_path_list(cls):
        return os.environ.get('KUMAOCHE_CONFIG_PATH', os.getcwd() + '/kumaoche_config.yml').split(':')

    @classmethod
    def all_roles(cls, file_path_list=None):
        loaded_yaml = cls.yaml_load(file_path_list)
        parsed_yaml = cls.parse_version(loaded_yaml)
        loaded_roles = parsed_yaml.get('roles', {})
        loaded_role_keys = list(loaded_roles.keys())

        # 順序を維持した重複削除
        return sorted(set(loaded_role_keys), key=loaded_role_keys.index)

    @classmethod
    def find(cls, role: str, file_path_list=None):
        loaded_yaml = cls.yaml_load(file_path_list)
        parsed_yaml = cls.parse_version(loaded_yaml)
        loaded_roles = parsed_yaml.get('roles', {})
        loaded_role_keys = list(loaded_roles.keys())

        if role not in loaded_role_keys:
            print(f'Target role "{role}" is not exist.')
            sys.exit()

        return ConfigPack(loaded_roles.get(role, {}))

    @classmethod
    def yaml_load(cls, file_path_list=None):
        if file_path_list is None:
            file_path_list = cls.file_path_list()

        # ファイル存在確認
        for path in file_path_list:
            if not os.path.exists(path):
                print(f'Target config file "{path}" is not exist.')
                sys.exit()

        loaded_yaml = {}
        for file_path in file_path_list:
            with open(file_path) as file:
                loaded_yaml = {**loaded_yaml, **yaml.safe_load(file)}

        return loaded_yaml

    @classmethod
    def parse_version(cls, loaded_yaml):
        version = int(loaded_yaml.get('version', 1))

        if version == 1:
            return loaded_yaml
        else:
            print(f'Yaml version "{version}" is invalid.')
            sys.exit()



# -*- coding: utf-8 -*-

from .exec_env import ExecEnv
from ..runner import Runner
from ..config import DockerConfig


class Docker(ExecEnv):
    def __init__(self, config: DockerConfig, variable: {}, runner: Runner):
        self.__config = config
        self.__default_var = variable
        self.__runner = runner

    def name(self):
        return self.__config.name

    def var_assign(self, text: str, append_var=None):
        if append_var is None:
            append_var = {}

        if len(self.__default_var) + len(append_var) > 0:
            return text.format(**self.__default_var, **append_var)

        return text

    def exec(self, command: str, work_dir='', append_var=None):
        if append_var is None:
            append_var = {}

        if work_dir != '':
            work_dir = self.__runner.path_filter(self.var_assign(work_dir))
            cmd = self.var_assign(command, append_var)
            return self.__runner.run(f'cd {work_dir} && {cmd}')

        return ''

    def build(self):
        return self.exec(self.__config.build, work_dir=self.__config.work_dir)

    def up(self):
        return self.exec(self.__config.up, work_dir=self.__config.work_dir)

    def down(self):
        return self.exec(self.__config.down, work_dir=self.__config.work_dir)

    def ps(self):
        return self.exec(self.__config.ps, work_dir=self.__config.work_dir)

    def run(self, command: str, work_dir='', container=''):
        if work_dir == '':
            work_dir = self.__config.work_dir

        if container == '':
            container = self.__config.container

        append_var = {'container': container, 'command': command}
        cmd = self.var_assign(command, append_var)

        if container != '':
            return self.exec(self.__config.run, work_dir=work_dir, append_var={'container': container, 'command': cmd})

        return ''



# -*- coding: utf-8 -*-

from kumaoche.runner import Runner


class MocRunner(Runner):
    @classmethod
    def run(cls, command: str):
        return command

    @classmethod
    def path_filter(cls, path: str):
        return path



# -*- coding: utf-8 -*-

from ..exec_env import ExecEnvConfig, ShellConfig, DockerConfig


class ServiceConfig(object):
    def __init__(self, parsed_yaml: {}, key: str):
        configs = parsed_yaml.get(key, {})
        env_name = configs.get('env', {}).get('name', '')

        if env_name == 'shell':
            self.env = ShellConfig(configs, 'env')
        elif env_name == 'docker':
            self.env = DockerConfig(configs, 'env')
        else:
            self.env = ExecEnvConfig(configs, 'env')

        self.run = configs.get('run', '')
        self.setup = configs.get('setup', '')
        self.update = configs.get('update', '')



# -*- coding: utf-8 -*-

import unittest
from kumaoche.service import DB
from ..exec_env import MocEnv
from ..config import StubConfig


class TestDb(unittest.TestCase):
    def setUp(self):
        self.config = StubConfig.find('variable_assign_test_role')
        self.empty_config = StubConfig.find('empty_role')
        self.env = MocEnv(self.config.variable)
        self.db = DB(self.env, self.config.db)
        self.empty_db = DB(self.env, self.empty_config.db)
        self.src_text = 'git_host:{git_host},git_org:{git_org},git_repo:{git_repo},db_host:{db_host},db_port:{db_port},db_user:{db_user},db_database:{db_database}'
        self.dst_text = 'git_host:github.com,git_org:kumak1,git_repo:kumaoche,db_host:db,db_port:3306,db_user:root,db_database:db'

    def test_run(self):
        self.assertEqual(f"work_dir:,command:db run test_command", self.db.run("test_command"))
        self.assertEqual(f"work_dir:,command:db run {self.dst_text}", self.db.run(self.src_text))
        self.assertEqual(f"work_dir:,command:", self.empty_db.run(self.src_text))

    def test_setup(self):
        self.assertEqual(f"work_dir:,command:db setup {self.dst_text}", self.db.setup())
        self.assertEqual(f"work_dir:,command:", self.empty_db.setup())

    def test_update(self):
        self.assertEqual(f"work_dir:,command:db update {self.dst_text}", self.db.update())
        self.assertEqual(f"work_dir:,command:", self.empty_db.update())


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from .exec_env_config import ExecEnvConfig


class StringBuilderConfig(ExecEnvConfig):
    def __init__(self, parsed_yaml: {}, key: str):
        super().__init__(parsed_yaml, key)



# -*- coding: utf-8 -*-

from .exec_env import ShellConfig, DockerConfig, StringBuilderConfig
from .service import GitConfig, DbConfig, PackageManagerConfig


class ConfigPack(object):
    def __init__(self, parsed_yaml):
        if parsed_yaml is None:
            parsed_yaml = {}

        self.variable = self.assign_dict(parsed_yaml, 'variable')
        self.string_builder = StringBuilderConfig(parsed_yaml, 'string_builder')
        self.shell = ShellConfig(parsed_yaml, 'shell')
        self.docker = DockerConfig(parsed_yaml, 'docker')
        self.git = GitConfig(parsed_yaml, 'git')
        self.db = DbConfig(parsed_yaml, 'db')
        self.php = PackageManagerConfig(parsed_yaml, 'php')
        self.ruby = PackageManagerConfig(parsed_yaml, 'ruby')
        self.node = PackageManagerConfig(parsed_yaml, 'node')

    @classmethod
    def assign_dict(cls, parsed_yaml: {}, key: str):
        configs = parsed_yaml.get(key, {})

        dictionary = {}
        for key in configs.keys():
            dictionary.setdefault(key, configs.get(key, ''))

        return dictionary



import unittest
from kumaoche.config import GitConfig


class TestGitConfig(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'git': {
                'run': '0',
                'setup': '1',
                'update': '2',
                'env': {
                    'name': '2.0',
                    'work_dir': '2.1',
                    'run': '2.2',
                },
                'repo_dir': '3',
            }
        }
        config = GitConfig(parsed_yaml, 'git')
        self.assertEqual('0', config.run)
        self.assertEqual('1', config.setup)
        self.assertEqual('2', config.update)
        self.assertEqual('2.0', config.env.name)
        self.assertEqual('2.1', config.env.work_dir)
        self.assertEqual('2.2', config.env.run)
        self.assertEqual('3', config.repo_dir)

        config = GitConfig({}, 'git')
        self.assertEqual('', config.run)
        self.assertEqual('', config.setup)
        self.assertEqual('', config.update)
        self.assertEqual('', config.env.name)
        self.assertEqual('', config.env.work_dir)
        self.assertEqual('', config.env.run)
        self.assertEqual('', config.repo_dir)



if __name__ == '__main__':
    unittest.main()



import unittest
from kumaoche.config import DockerConfig


class TestDockerConfig(unittest.TestCase):
    def test_init(self):
        parsed_yaml = {
            'docker': {
                'run': '0',
                'name': '1',
                'work_dir': '2',
                'container': '3',
                'ps': '4',
                'build': '5',
                'up': '6',
                'down': '7',
            }
        }
        config = DockerConfig(parsed_yaml, 'docker')
        self.assertEqual('0', config.run)
        self.assertEqual('1', config.name)
        self.assertEqual('2', config.work_dir)
        self.assertEqual('3', config.container)
        self.assertEqual('4', config.ps)
        self.assertEqual('5', config.build)
        self.assertEqual('6', config.up)
        self.assertEqual('7', config.down)

        config = DockerConfig({}, 'docker')
        self.assertEqual('', config.run)
        self.assertEqual('', config.name)
        self.assertEqual('', config.work_dir)
        self.assertEqual('', config.container)
        self.assertEqual('', config.ps)
        self.assertEqual('', config.build)
        self.assertEqual('', config.up)
        self.assertEqual('', config.down)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from .service_config import ServiceConfig


class PackageManagerConfig(ServiceConfig):
    def __init__(self, parsed_yaml: {}, key: str):
        super().__init__(parsed_yaml, key)

        configs = parsed_yaml.get(key, {})
        self.test = configs.get('test', '')



# -*- coding: utf-8 -*-

from .service_config import ServiceConfig


class GitConfig(ServiceConfig):
    def __init__(self, parsed_yaml: {}, key: str):
        super().__init__(parsed_yaml, key)

        configs = parsed_yaml.get(key, {})

        self.repo_dir = configs.get('repo_dir', '')



# -*- coding: utf-8 -*-

from .exec_env import ExecEnv


class DisableEnv(ExecEnv):
    def name(self):
        return 'disable'

    def var_assign(self, text: str, append_var=None):
        return text

    def run(self, command: str, work_dir=''):
        return



# -*- coding: utf-8 -*-

from ..config import PackageManagerConfig
from ..exec_env import ExecEnv
from .service import Service


class PackageManager(Service):
    def __init__(self, env: ExecEnv, config: PackageManagerConfig):
        self.__env = env
        self.__config = config

    def run(self, command: str):
        var_command = {'command': self.__env.var_assign(command)}
        return self.__env.run(self.__env.var_assign(self.__config.run, var_command))

    def setup(self):
        return self.__env.run(self.__env.var_assign(self.__config.setup))

    def update(self):
        return self.__env.run(self.__env.var_assign(self.__config.update))

    def test(self, suffix=''):
        cmd = self.__env.var_assign(self.__config.test)

        if suffix != '':
            suffix = ' ' + self.__env.var_assign(suffix)

        return self.__env.run(f'{cmd}{suffix}')



# -*- coding: utf-8 -*-

import os

from kumaoche.config import ConfigParser, ConfigPack


class StubConfig(object):
    @classmethod
    def find(cls, role: str):
        file_path_list = [os.path.dirname(__file__) + '/stub_config.yml']
        return ConfigParser.find(role, file_path_list)



# -*- coding: utf-8 -*-

from .exec_env import ExecEnv
from ..runner import Runner
from ..config import ShellConfig


class Shell(ExecEnv):
    def __init__(self, config: ShellConfig, variable: {}, runner: Runner):
        self.__config = config
        self.__default_var = variable
        self.__runner = runner

    def name(self):
        return self.__config.name

    def var_assign(self, text: str, append_var=None):
        if append_var is None:
            append_var = {}

        if len(self.__default_var) + len(append_var) > 0:
            return text.format(**self.__default_var, **append_var)

        return text

    def run(self, command: str, work_dir=''):
        if work_dir == '':
            work_dir = self.__runner.path_filter(self.var_assign(self.__config.work_dir))

        if work_dir != '':
            work_dir = f'cd {work_dir} && '

        cmd = self.var_assign(command)
        cmd = self.var_assign(self.__config.run, {'command': cmd})

        return self.__runner.run(f'{work_dir}{cmd}')



# -*- coding: utf-8 -*-

import unittest
from kumaoche.exec_env import Shell
from ..config import StubConfig
from ..runner import MocRunner


class TestShell(unittest.TestCase):
    def setUp(self):
        self.config = StubConfig.find('variable_assign_test_role')
        self.empty_config = StubConfig.find('empty_role')
        self.env = Shell(self.config.shell, self.config.variable, MocRunner)
        self.empty_env = Shell(self.empty_config.shell, self.empty_config.variable, MocRunner)
        self.src_text = 'git_host:{git_host},git_org:{git_org},git_repo:{git_repo},db_host:{db_host},db_port:{db_port},db_user:{db_user},db_database:{db_database}'
        self.dst_text = 'git_host:github.com,git_org:kumak1,git_repo:kumaoche,db_host:db,db_port:3306,db_user:root,db_database:db'

    def test_run(self):
        self.assertEqual(f"cd shell work_dir {self.dst_text} && shell run test_command", self.env.run("test_command"))
        self.assertEqual(f"cd shell work_dir {self.dst_text} && shell run {self.dst_text}", self.env.run(self.src_text))
        self.assertEqual("", self.empty_env.run(self.src_text))

    def test_var_assign(self):
        self.assertEqual(self.dst_text, self.env.var_assign(self.src_text))
        self.assertEqual('', self.env.var_assign(''))
        self.assertEqual('changed', self.env.var_assign('{var_test}', {'var_test': 'changed'}))
        self.assertEqual(self.src_text, self.empty_env.var_assign(self.src_text))

    def test_name(self):
        self.assertEqual("shell", self.env.name())


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

from kumaoche.exec_env.exec_env import ExecEnv


class MocEnv(ExecEnv):
    def __init__(self, variable: {}):
        self.__default_var = variable

    def name(self):
        return ''

    def var_assign(self, text: str, append_var=None):
        if append_var is None:
            append_var = {}

        if len(self.__default_var) + len(append_var) > 0:
            return text.format(**self.__default_var, **append_var)

        return text

    def run(self, command: str, work_dir=''):
        return f'work_dir:{work_dir},command:{command}'
